/**
 *  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance
 *  with the License. A copy of the License is located at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  or in the 'license' file accompanying this file. This file is distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES
 *  OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions
 *  and limitations under the License.
 */
import * as lambda from 'aws-cdk-lib/aws-lambda';
import * as s3 from 'aws-cdk-lib/aws-s3';
import { Construct } from 'constructs';
import { BaseClass } from '../../../common/base-class';
/******************************************************************************
 *                        PROPS FOR NEW CONSTRUCT
 *****************************************************************************/
/**
 * Properties for creating a CDK BDA construct.
 */
export interface BedrockDataAutomationProps {
    /**
     * - Optional. The S3 bucket
     * for input data used by the Bedrock Data Automation process.
     * If not provided, a new bucket will be created.
     */
    readonly inputBucket?: s3.IBucket;
    /**
     * - Optional. The S3 bucket for storing
     * output files generated by the Bedrock Data Automation process.
     */
    readonly outputBucket?: s3.IBucket;
    /**
     * - Optional. Indicates whether a custom
     * Bedrock Data Automation blueprint is required. If true, the necessary resources will be created.
     *
     * @default - false
     */
    readonly isCustomBDABlueprintRequired?: boolean;
    /**
     * - Optional. Indicates whether a Bedrock Data
     * Automation project is required. If true, the necessary resources will be created.
     *
     * @default - false
     */
    readonly isBDAProjectRequired?: boolean;
    /**
     * - Optional. Indicates whether a Bedrock Data
     * Automation invocation is required. If true, the necessary resources will be created.
     *
     * @default - false
     */
    readonly isBDAInvocationRequired?: boolean;
    /**
     * - Optional. Indicates whether the status of the
     * Bedrock Data Automation process is required. If true, the necessary resources will be created.
     *
     * @default - false
     */
    readonly isStatusRequired?: boolean;
}
/******************************************************************************
 *                        NEW CONSTRUCT DEFINITION
 *****************************************************************************/
/**
 * Class to create a BDA pattern with CDK.
 */
export declare class BedrockDataAutomation extends BaseClass {
    /**
     * The S3 bucket for input data used by the Bedrock Data Automation process.
     * IMPORTANT: If isCustomBDABlueprintRequired or isBDAInvocationRequired are set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaInputBucket?: s3.IBucket;
    /**
     * The S3 bucket for output data generated by the Bedrock Data Automation process.
     * IMPORTANT: If isBDAInvocationRequired is set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaOutputBucket?: s3.IBucket;
    /**
     * The Lambda function responsible for creating the Bedrock Data Automation blueprint.
     * IMPORTANT: If isCustomBDABlueprintRequired is set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaBlueprintLambdaFunction?: lambda.Function;
    /**
     * The Lambda function responsible for handling the Bedrock Data Automation project.
     * IMPORTANT: If isBDAProjectRequired is set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaProjectFunction?: lambda.Function;
    /**
     * The Lambda function responsible for invoking the Bedrock Data Automation process.
     * IMPORTANT: If isBDAInvocationRequired is set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaInvocationFunction?: lambda.Function;
    /**
     * The Lambda function responsible for checking the status of the Bedrock Data Automation process.
     * IMPORTANT: If isStatusRequired is set to false in Pattern Construct Props,
     * this property will be undefined
     */
    readonly bdaResultStatusFunction?: lambda.Function;
    /**
     * The AWS Lambda Powertools layer used in the Lambda functions.
     */
    readonly powertoolsLayer: lambda.ILayerVersion;
    /**
     * The Boto3 layer used in the Lambda functions for AWS SDK interactions.
     */
    readonly boto3Layer: lambda.LayerVersion;
    constructor(scope: Construct, id: string, props: BedrockDataAutomationProps);
    /**
     * Handles the creation or retrieval of an S3 bucket.
     *
     * @param existing_bucket - An optional existing S3 bucket to use.
     * @param type - A string indicating the type of bucket (e.g., 'input' or 'output').
     * @returns The existing bucket if provided, or a newly created S3 bucket.
     */
    private handleS3Bucket;
}
